import { BaseService } from "./base.service";
import { Page } from "tns-core-modules/ui/page/page";
import { topmost } from "tns-core-modules/ui/frame";
import { DockLayout } from "tns-core-modules/ui/layouts/dock-layout/dock-layout";
import { Color } from "tns-core-modules/color/color";
import { StackLayout } from "tns-core-modules/ui/layouts/stack-layout/stack-layout";
import { Label } from "tns-core-modules/ui/label/label";
import { View } from "tns-core-modules/ui/core/view/view";
import { TextField } from "tns-core-modules/ui/text-field/text-field";
import { AbsoluteLayout } from "tns-core-modules/ui/layouts/absolute-layout/absolute-layout";
import { GridLayout, ItemSpec } from "tns-core-modules/ui/layouts/grid-layout/grid-layout";

declare var NSLineBreakMode;

import * as application from "application";
import * as fs from "file-system";
import { ErrorResolver } from "../module.language/error.resolver";
import { Image } from "tns-core-modules/ui/image/image";
import { ViewContainerRef } from "@angular/core";
import * as observableModule from "data/observable";
import { observe } from "tns-core-modules/ui/gestures/gestures";
import { Observable, PropertyChangeData, Size } from "tns-core-modules/ui/editable-text-base/editable-text-base";
import { LayoutBase } from "tns-core-modules/ui/layouts/layout-base";
import { setTimeout, clearInterval } from "tns-core-modules/timer/timer";

import * as timer from "timer";


export class BaseComponent {
    protected componentName   : string;

    protected border;

    protected pageI;

    protected runningErrors: Array<{view: any, height: number}>;

    protected resolver: ErrorResolver;

    constructor(protected baseService: BaseService, private optionals?:{ componentName?: string, vcRef?: ViewContainerRef}){ 
        if(optionals){
            if(optionals.componentName === undefined){
                this.componentName = this.constructor.name
            }
            else{
                this.componentName = optionals.componentName;
            }
        }
        else{
            this.componentName = this.constructor.name;
        }

        this.runningErrors  = new Array();
        this.resolver       = this.baseService.getResolver();
    }

    protected l(line_number: number): string{
        return this.baseService.getLine(line_number);
    }

    protected getConfigString(key: string, defaultValue?: string): string{
		return this.baseService.getConfigString(key, defaultValue);
	}

	protected getConfigNumber(key: string, defaultValue?: number): number{
		return this.baseService.getConfigNumber(key, defaultValue);
	}

	protected getConfigBoolean(key: string, defaultValue?: boolean): boolean{
		return this.baseService.getConfigBoolean(key, defaultValue);
	}

	protected setConfigString(key: string, value: string){
		this.baseService.setConfigString(key, value);
	}

	protected setConfigNumber(key: string, value: number){
		this.baseService.setConfigNumber(key, value);
	}

	protected setConfigBoolean(key: string, value: boolean){
		this.baseService.setConfigBoolean(key, value);
    }
    
    validatePassword(password: string){
        return this.baseService.validatePassword(password);
    }

    validatePostcode(postcode: string){
        return this.baseService.validatePostcode(postcode);
    }

    validateNoNumber(exp: string){
        return this.baseService.validateNoNumber(exp);
    }

    checkEmpty(exp: string){
        return this.baseService.checkEmpty(exp);
    }

    protected  log(message?: any, showOnConsole?/*default: true*/: boolean){
        if(showOnConsole === undefined){
            showOnConsole = true;
        }
        
        this.baseService.log(message, this.componentName, showOnConsole)
    }

    protected  navigate(dest: string, transition?:{name: "fade" | "flip" | "flipRight" | "flipLeft" | "slide" | "slideLeft" | "slideRight" | "slideTop" | "slideBottom", duration: number, curve: "ease" | "easeIn" | "easeInOut" | "easeOut" | "linear" | "spring"}){
        this.baseService.navigate(dest, transition);
    }
    
    protected resolveError(errorCode: number): string{
        return this.resolver.resolveError(errorCode);
    }

    protected showImage(image: Image){
        if(application.ios){
            this.baseService.showImage(image);
        }
        else{
            let folder: fs.Folder = fs.knownFolders.documents().getFolder("imageTemp");
            if(image.imageSource.saveToFile(folder.path + "img", "png")){
                let path: string = folder.path + "img.png";
                this.baseService.showImage({path: path, vcRef: this.optionals.vcRef});
            }
        }
    }
        
    
}
