import { Injectable, EventEmitter } from '@angular/core';
import { ControllerService, ICMDHandler, IMessage } from '../module.controller/controller.service';
import { ErrorResolver } from "../module.language/error.resolver"
import * as app from "application";
import { ImageSource } from "tns-core-modules/image-source/image-source";

import * as fs from "file-system"
import { User } from '../module.types/types.declarations';


@Injectable()
export class BaseService implements ICMDHandler {

	handlername: string;

	show;

	debug	: boolean;

	currentUser: User;
	

//********************************************************************************************************
	constructor(protected controllerService: ControllerService, handlername: string) {
		if(handlername) {
			this.handlername = handlername;
			this.controllerService.registerHandler(this);
		}
		this.debug 		 = false;
		this.currentUser = this.getCurrentUser();
		if(this.currentUser != undefined && this.currentUser){
			this.onSetCurrentUser();
		}
	 }


	//********************************************************************************************************
	// MessageController
	//********************************************************************************************************
	public inbox(msg: IMessage){
		switch (msg.cmd) {
			case "setCurrentUser":
				this.currentUser = msg.parameter.user;
				console.log("setCurrentUser",this.currentUser)
				this.onSetCurrentUser();
				break;
		
			default:
				break;
		}
	}

	//********************************************************************************************************
	/** Called when the currentUser is set. */
	protected onSetCurrentUser(){}

	//********************************************************************************************************
	/** Navigate to destination page. */
	public navigate(dest: string, transition?:{name: "fade" | "flip" | "flipRight" | "flipLeft" | "slide" | "slideLeft" | "slideRight" | "slideTop" | "slideBottom", duration: number, curve: "ease" | "easeIn" | "easeInOut" | "easeOut" | "linear" | "spring"}, clearHistory: boolean=false){
		this.controllerService.callHandler('router', {cmd:'navigate', parameter:{dest:dest, clearHistory: false, transition:transition}});
	}

	/** Return to previous page in navigation history. */
	public goBack(){
		this.controllerService.callHandler('router', {cmd: 'back'});
	}
	
	/** Return a line (string) from language module in the device language. */
	getLine(number: number):string{
		return this.controllerService.callHandler("language", {cmd:"getLine", parameter: {number: number}}, false);
	}

//error resolver 
//********************************************************************************************************
	/** -deprecated- Returns a Error Resolver instance. */
	getResolver():ErrorResolver{
		return new ErrorResolver();
	}

//get Picture
//********************************************************************************************************
	/** Uses picture module to get a picture from camera or gallery. */
	getPicture(){
		return this.controllerService.callHandler("camera", {cmd:"getPicture", parameter:{userCut:"false"}});
	}

//validate password
//********************************************************************************************************
	/** Validation: Check if password conforms to given standards. */
	validatePassword(password:string){
		return this.controllerService.callHandler("validator", {cmd:"validatePassword", parameter:{password:password}});
	}

//validate containing no number 
//********************************************************************************************************
	/** Validation: Check if string does not include any numbers. */
	validateNoNumber(str:string){
		return this.controllerService.callHandler("validator", {cmd:"validateNoNumber", parameter:{str:str}});
	}

//validate containing no number 
//********************************************************************************************************
	/** Validation: valid german postcode. */
	validatePostcode(postcode:string){
		return this.controllerService.callHandler("validator", {cmd:"validatePostcode", parameter:{postcode:postcode}});
	}

//Check if empty
//********************************************************************************************************
	/** Validation: Check if string is empty. */
	checkEmpty(exp){
		return this.controllerService.callHandler("validator", {cmd:"checkEmpty", parameter:{exp: exp}});
	}

	//get the user with the given user_id
	//********************************************************************************************************
	public getUser(user_id: string, user?: User): Promise<User> {
		return this.controllerService.callHandler("user", {cmd:"getUser", parameter:{user_id: user_id, user: user}});
	}

	//get the user with the given user_id from local storage
	//********************************************************************************************************
	public getUserLocal(user_id: string): User {
		return this.controllerService.callHandler("user", {cmd:"getUserLocal", parameter:{user_id: user_id}});
	}

	//get the current loggedin user from the vault
	//********************************************************************************************************
	public getCurrentUser(): User {
		return this.controllerService.callHandler("vault", {cmd:"getCurrentUser"});
	}

	//save the current loggedin user from the vault
	//********************************************************************************************************
	public saveCurrentUser(): User {
		return this.controllerService.callHandler("user", {cmd:"saveUserLocal", parameter:{user: this.currentUser}});
	}

	public pushNote(title: string, body: string){
		return this.controllerService.callHandler("push", {cmd:"note", parameter:{title:title, body: body}});
	}


	//animate, easing functions for number values
	//********************************************************************************************************
	/** Animate actual size of an layout object. (Changes TDB) */
	animate(callback, b, c, d, func){
		return this.controllerService.callHandler("animation", {cmd:"animate", parameter:{callback: callback, begin: b, change: c, duration:d, function:func}});
	}

	/** Log a message to the console and to current logfile. */
	log(message?: any, sender?: string, showOnConsole?/*default: true*/: boolean){
		if(showOnConsole === undefined){
			showOnConsole = true;
		}
		if(sender === undefined){
			sender = this.constructor.name;
		}
		if(showOnConsole){
			console.log("[" + sender + "]: " +message);
		}

		//TODO: Log
	}	  

	protected saveToLocal(data: string, filepath: string){
		this.controllerService.callHandler("localstorage",{cmd: "saveToLocal", parameter: {data: data, filepath: filepath}});
	}

	protected loadFromLocal(filepath: string){
		return this.controllerService.callHandler("localstorage",{cmd: "loadFromLocal", parameter: {filepath: filepath}});
	}

	protected loadAllFilesFromPath(path: string): Array<string> {
		return this.controllerService.callHandler("localstorage",{cmd: "loadAllFilesFromPath", parameter: {path: path}});
	}

	/** Gets a string from the local config. */
	getConfigString(key: string, defaultValue?: string): string{
		if(defaultValue){
			return this.controllerService.callHandler("config", {cmd: "getConfig", parameter:{type: "string", key: key, defaultValue: defaultValue}});
		}
		else{
			return  this.controllerService.callHandler("config", {cmd: "getConfig", parameter:{type: "string", key: key}});
		}
	}

	/** Gets a number from the local config. */
	getConfigNumber(key: string, defaultValue?: number): number{
		if(defaultValue){
			return this.controllerService.callHandler("config", {cmd: "getConfig", parameter:{type: "number", key: key, defaultValue: defaultValue}});
		}
		else{
			return  this.controllerService.callHandler("config", {cmd: "getConfig", parameter:{type: "number", key: key}});
		}
	}

	/** Gets a boolean from the local config. */
	getConfigBoolean(key: string, defaultValue?: boolean): boolean{
		if(defaultValue){
			return this.controllerService.callHandler("config", {cmd: "getConfig", parameter:{type: "boolean", key: key, defaultValue: defaultValue}});
		}
		else{
			return  this.controllerService.callHandler("config", {cmd: "getConfig", parameter:{type: "boolean", key: key}});
		}
	}

	/** Sets a string in the local config. */
	setConfigString(key: string, value: string){
		this.controllerService.callHandler("config", {cmd: "setConfig", parameter:{type: "string", key: key, value: value}});
	}

	/** Sets a number in the local config. */
	setConfigNumber(key: string, value: number){
		this.controllerService.callHandler("config", {cmd: "setConfig", parameter:{type: "number", key: key, value: value}});
	}

	/** Sets a boolean in the local config. */
	setConfigBoolean(key: string, value: boolean){
		this.controllerService.callHandler("config", {cmd: "setConfig", parameter:{type: "boolean", key: key, value: value}});
	}

	/** - deprecated - */
	public vault(filename: string, content){
        this.controllerService.callHandler("vault", {cmd:"append", parameter:{filename:filename, content: content}});
	}
	
	/** Shows the image in an imageviewer. */
	showImage(image){
		this.controllerService.callHandler("camera", {cmd:"showImage", parameter:{image: image, path: image.path, vcRef: image.vcRef}}, false);
	}

	/** Returns a random string with given length. */
	randomString(length: number) {
		let text: string = "";
		let possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
		for(let i = 0; i < length; i++) {
			text += possible.charAt(Math.floor(Math.random() * possible.length));
		}
		return text;
	}
	
	/**Send message to the server */
	sendMessageToNetwork(cmd: string, parameter: any, attachments?: Array<any>): Promise<any> {
		return new Promise((resolve, reject)=>{
			this.controllerService.callHandler("network", {cmd:"sendMessage:"+cmd, parameter:parameter, attachments: attachments})
			.then(
				res=>{
					resolve(res);
				},err=>{
					console.log("Network error: ",err);
					resolve({error: 400, errMsg: err});
				}
			)
		})
	}
}	

