import { EventEmitter, Injectable } from '@angular/core';
import { Observable } from 'tns-core-modules/data/observable/observable';
import * as fs from "file-system";

import { Attachment, LoadingStatus} from '../module.types/types.declarations';


@Injectable()
export class ControllerService {

//********************************************************************************************************
// Class Variables
//********************************************************************************************************
	private handlerList = new Map<string,ICMDHandler>();
	private hasLogger:boolean;
	private logFunc;
	private loadingStatus: LoadingStatus;
	public  globals;
	
//constructor
//********************************************************************************************************
	constructor() {
		this.globals = {};
		this.hasLogger = false;
	}

//********************************************************************************************************
// Class Functions
//********************************************************************************************************

//register a new handler with name and a function to call
//********************************************************************************************************
	registerHandler(handler: ICMDHandler){
		if(!this.handlerList.has(handler.handlername)) {
			this.handlerList.set(handler.handlername, handler);
		}
	}

//register a logger which is used to log every message send via messagecontroller
//********************************************************************************************************
	registerLogger(logFunc: {():void}){
		this.hasLogger = true;
		this.logFunc = logFunc;
	}

	callHandler(receivingHandler: string, msg: IMessage, shouldBeLogged?: boolean, loadingStatus?: LoadingStatus): any {
		if(this.hasLogger === true && (receivingHandler === "logger" && (msg.cmd === "logMessage" || msg.cmd === "logError")) === false){
			if(shouldBeLogged === undefined){
				this.logFunc(receivingHandler, {cmd: msg.cmd, parameter: msg.parameter});
			}	
		}
		if(loadingStatus){
			this.loadingStatus = loadingStatus;
		}
		if(this.handlerList.has(receivingHandler)){
			return this.handlerList.get(receivingHandler).inbox(msg,this.loadingStatus);
		}
		else{
			this.handlerList.forEach((handler: ICMDHandler) =>{
				handler.inbox(msg);
			})
		}
		
	}
}

//********************************************************************************************************
export interface ICMDHandler {
	handlername: string;

	inbox(msg: IMessage, loadingStatus?: LoadingStatus);
}

//********************************************************************************************************
export interface IMessage {
	cmd:  			string;
	parameter?:  	any;
	attachments?:	Array<Attachment>;
	loadingStatus?: LoadingStatus;
}
