import { EventEmitter } from '@angular/core';
import { ImageSource } from "tns-core-modules/image-source/image-source";
import * as fs from "file-system";
import { ObservableArray } from "tns-core-modules/data/observable-array/observable-array";
import { Color } from "tns-core-modules/color/color";
import { isAndroid, isIOS, device, screen } from "platform";
import { setTimeout } from "tns-core-modules/timer/timer";


export enum Login_Types{
	email,
	acces_token,
	social
}

export enum Sex{
	female = "w",
	male = "m"
}

export class User {
    public user_id          : string;
	public user_token		: string;
    public firstname        : string;
    public lastname         : string;
    public user_type        : string;
    public street           : string;
    public place            : string;
    public postcode         : string;
    public sex             	: Sex;
    public birthdate        : Date;
    public profilePicture   : Base64ImageSource;
    public password         : string; 
	public additional_info  : string;
	
	public pic_path			: string;
	public last_access		: string;

	public chats			: Array<Chat>;
	private _birthday		: string;

    constructor(user?: User){
		this.setFromUser(user);
	}

	public setFromUser(user: User){
		
		this.birthdate = new Date();
		this._birthday = dateStr(this.birthdate);

		this.profilePicture = new Base64ImageSource();
		this.chats			= new Array(0);
		if(user){
			this.firstname        	= user.firstname;
			this.lastname         	= user.lastname;
			this.user_id          	= user.user_id;
			this.user_type        	= user.user_type;
			this.street           	= user.street;
			this.place            	= user.place;
			this.postcode         	= user.postcode;
			this.sex             	= user.sex;
			this.birthdate			= new Date(user.birthdate);
			this._birthday			= dateStr(this.birthdate);
			if(user.profilePicture && user.profilePicture.Data){
				this.profilePicture.Data = user.profilePicture.Data;
			}
			this.password         	= user.password;
			this.additional_info  	= user.additional_info;

			this.user_token 		= user.user_token;
			this.pic_path			= user.pic_path;
			this.last_access		= user.last_access;
		}
	}


	set pic(val: string){
		this.profilePicture.Data = val;
	}

    get birthday(): string{
		return dateStr(this.birthdate);
    }

	set birthday(value: string){
		this._birthday = value;
		let split = value.split(".")
		if(split.length==3){
			this.birthdate.setFullYear(Number.parseInt(split[2]),Number.parseInt(split[1])-1,Number.parseInt(split[0]));
		}
    }

	public get placeAndPostcode(): string{
		return this.postcode + " " + this.place;
	}
	
	get address(): string{
		return this.street + " - " + this.postcode + " - " + this.place;
	}

	get sex_string(){
		if(this.sex === Sex.male){
			return "männlich";
		}
		else{
			return "weiblich";
		}
	}

	public toJSON(){
		let chats = new Array(0);
		if(this.chats){
			for(let i=0;i<this.chats.length;i++){
				chats.push({chat_id: this.chats[i].chat_id});
			}
		}
		return Object.assign({}, this, {chats: chats, _birthday: undefined, _sex: undefined});
	}
}


//Chat
//********************************************************************************************************
/** Chat object */
export class Chat{
	chat_id		: string;
	name		: string;
	user		: Array<User>
	messages	: Array<ChatMessage>;
	last_access : string;

	constructor(chat?:Chat){
		this.setFromChat(chat);
	}

	public setFromChat(chat: Chat){
		this.user 	  = new Array(0);
		this.messages = new Array(0);

		if(chat){
			this.chat_id 	 = chat.chat_id;
			this.name		 = chat.name;
			this.last_access = chat.last_access;
			if(chat.user){
				for(let i = 0; i<chat.user.length; i++){
					if(chat.user[i] instanceof User){
						this.user.push(chat.user[i])
					}else{
						this.user.push(new User(chat.user[i]))
					}
				}
			}
			if(chat.messages){
				for(let i = 0; i<chat.messages.length; i++){
					if(chat.messages[i] instanceof ChatMessage){
						this.messages.push(chat.messages[i]);
					}else{
						this.messages.push(new ChatMessage(chat.messages[i]));
					}
				}
			}
		}
	}

	// public setStatus(message_id: string, state: ChatMessageState){
	// 	let msg = this.messages.filter(element=> element.dataset_id === message_id)
	// 	if(msg.length === 1){
	// 		msg[0].status = state;
	// 	}
	// }

	/** Returns a string of all names, seperated with a comma. */
	getNames(){
		let names: string="";
		if(this.user.length > 0){
			names = this.user[0].firstname + " " + this.user[0].lastname;
			for(let i = 1 ; i < this.user.length ; i++){
				names += ", " + this.user[i].firstname + " " + this.user[i].lastname;
			}
		}
		return names;
	}

	/** Gets the last member of the messages array. */
	get lastMessage(): ChatMessage{
		let result = this.messages[this.messages.length-1]
		if(!result){
			result = new ChatMessage();
			result.message = "Keine Nachrichten"
		}
		return result;
	}

	public toJSON() {
		return Object.assign({},{user: this.user, name: this.name, chat_id: this.chat_id, last_access: this.last_access});
	}
}

export enum ChatMessageState{
	pending,
	server_not_reachable,
	delivered,
	read
}

export class ChatMessage{
	chat_id			: string;
	user_id			: string;
	message			: string;
	timestamp		: Date;
	dataset_id		: string;
	status			: any;
	attachments		: string;
	
	picture			: Base64ImageSource;
	voiceMessage	: {file: string, duration: number, curTime: number};

	constructor(chatMessage?: ChatMessage){
		this.picture = new Base64ImageSource();
		if(chatMessage){
			this.chat_id 	 = chatMessage.chat_id;
			this.user_id 	 = chatMessage.user_id;
			this.message 	 = chatMessage.message;
			this.attachments = chatMessage.attachments;
			if(chatMessage.timestamp instanceof Date){
				this.timestamp = new Date(chatMessage.timestamp);
			}else{
				this.timestamp = parseDateTimeStr(chatMessage.timestamp);
			}
			this.dataset_id	= chatMessage.dataset_id;
			if(typeof chatMessage.status == "string" && chatMessage.status){
				this.status = JSON.parse(chatMessage.status);
			}else{
				this.status = chatMessage.status;
			}
		}
	}

	public hasVoiceMessage(): boolean {
		if(this.voiceMessage){
			return true;
		}
		return false;
	}

	public hasPicture(): boolean {
		if(this.picture.DataStr){
			return true;
		}
		return false;
	}
	
	/** Returns string containing hours:minutes. */
	public getDateStr(): string{
		return timeStr(this.timestamp);
	}

	public toJSON() {
		return {chat_id: this.chat_id, user_id: this.user_id, message: this.message, dataset_id: this.dataset_id, attachments: this.attachments, timestamp: this.timestamp, status: this.status}
	}   

	public getStatus(): ChatMessageState {
		let status;
		status = ChatMessageState.read;
		for(let key in this.status){ 
			let value = this.status[key];
			if( value == ChatMessageState.pending){
				status = ChatMessageState.pending;
				break;
			}else if(value == ChatMessageState.delivered && status != ChatMessageState.pending) {
				status = ChatMessageState.delivered;
			}
		}
		return status;
	}
}

export function parseDateTimeStr(dateTimeStr: string): Date {
	let split = dateTimeStr.split("T");
	let dateStr: string = "";
	let timeStr: string = "";
	if(split.length == 2){
		dateStr = split[0]
		timeStr = split[1]
	}else{
		split = dateTimeStr.split(" ");
		dateStr = split[0]
		timeStr = split[1]
	}
	let year   = Number.parseInt(dateStr.split("-")[0]);
	let month  = Number.parseInt(dateStr.split("-")[1]);
	let day	   = Number.parseInt(dateStr.split("-")[2]);
	let hour   = Number.parseInt(timeStr.split(":")[0]);
	let minute = Number.parseInt(timeStr.split(":")[1]);
	let sec	   = Number.parseInt(timeStr.split(":")[2]);
	return new Date(year,month-1,day,hour,minute,sec);
}

export function timeStr(date: Date): string {
	let hourStr   = date.getHours().toString();
	let minuteStr = date.getMinutes().toString();
	let secStr	  = date.getSeconds().toString();
	if(date.getHours()<10){
		hourStr = "0"+hourStr;
	}
	if(date.getMinutes()<10){
		minuteStr = "0"+minuteStr;
	}
	if(date.getSeconds()<10){
		secStr = "0"+secStr;
	}
	return hourStr+":"+minuteStr+":"+secStr;
}

export function dateStr(date: Date): string {
	let yearStr   = date.getFullYear().toString();
	let monthStr  = (date.getMonth()+1).toString();
	let dayStr	  = date.getDate().toString();
	if(date.getMonth()+1<10){ 
		monthStr = "0"+monthStr;
	}
	if(date.getDate()<10){
		dayStr = "0"+dayStr;
	}
	return dayStr+"."+monthStr+"."+yearStr;
}

export class Base64ImageSource {
	private _data:  	Promise<string> | string;
	private _dataStr:	string;
	private _image = 	null;

	public ImageSource: string;

	
	constructor(data?: string | Promise<string>){
		if(data){
			this.Data = data;
		}else{
			this.Data = "";
		}
	}

	public toJSON(){
		return undefined;// Object.assign({}, {_image: undefined, _data: undefined, _dataStr: undefined, ImageSource: undefined});
	}

	set Data(data: string | Promise<string>){
		this._data = data;
		if(typeof data === 'string' || data instanceof String){
			this.setImageSource(<string>data);
		}else{
			if(this._data){
				let p = <Promise<string>>this._data;
				p.then(
				res=>{
					this._data 	  = res;
					this.setImageSource(res);
				},err=>{
					console.log(err);
				});
			}
		}
	}

	get Data(): string | Promise<string> {
		return this._data;
	}

	get DataStr(): string {
		return this._dataStr;
	}

	set Image(image){
		this._image = image;
		this._image.src = this.ImageSource;
	}

	get Image() {
		return this._image;
	}

	private setImageSource(data: string){
		this._dataStr    = data;
		this.ImageSource = "data:image/png;base64,"+data;
		if(this._image){
			this._image.src = this.ImageSource;
		}
	}

}
